<?php
declare(strict_types=1);

namespace Catequil\Db;

use Doctrine\DBAL\Connection as BaseConnection;
use Doctrine\DBAL\Exception as DBALException;

use function array_fill;
use function array_keys;
use function array_map;
use function array_values;
use function count;
use function current;
use function implode;
use function in_array;
use function preg_replace;
use function sprintf;

/**
 * Class Connection
 *
 * @package Catequil\Db
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-db
 */
final class Connection extends BaseConnection
{
    /**
     * @param string $table
     * @param array $rows
     * @param array $columns
     *
     * @return int
     *
     * @throws DBALException
     */
    public function multiInsert(string $table, array $rows, array $columns = []): int
    {
        if (empty($columns)) {
            $columns = array_keys(current($rows));
        }

        $sql    = $this->createInsertSql($table, $columns, count($rows));
        $params = $this->unrollArray($rows);

        return $this->executeStatement($sql, $params);
    }

    /**
     * @param string $table
     * @param array $rows
     * @param array $columns
     * @param array $ignoreForUpdate
     *
     * @return int
     *
     * @throws DBALException
     */
    public function multiUpsert(string $table, array $rows, array $columns = [], array $ignoreForUpdate = []): int
    {
        if (empty($columns)) {
            $columns = array_keys(current($rows));
        }

        $sql = $this->createUpsertSql($table, $columns, $ignoreForUpdate, count($rows));
        $params = $this->unrollArray($rows);

        return $this->executeStatement($sql, $params);
    }

    /**
     * @param string $table
     * @param array $data
     * @param array $ignoreForUpdate
     *
     * @return int
     *
     * @throws DBALException
     */
    public function upsert(string $table, array $data, array $ignoreForUpdate = []): int
    {
        $columns = array_keys($data);

        $sql = $this->createUpsertSql($table, $columns, $ignoreForUpdate);
        $params = array_values($data);

        return $this->executeStatement($sql, $params);
    }


    /**
     * @param string $table
     * @param array $columns
     * @param int $rowCount
     *
     * @return string
     */
    private function createInsertSql(string $table, array $columns, int $rowCount = 1): string
    {
        /**
         * @noinspection SqlNoDataSourceInspection
         * @noinspection SqlResolve
         */
        return sprintf(
            'INSERT INTO `%s` (%s) VALUES %s',
            $table,
            implode(', ', array_map([$this, 'escapeMySqlIdentifier'], $columns)),
            implode(', ', array_fill(
                0,
                $rowCount,
                '(' . implode(', ', array_fill(0, count($columns), '?')) . ')'
            )),
        );
    }

    /**
     * @param string $table
     * @param array $columns
     * @param array $ignoreForUpdate
     * @param int $rowCount
     *
     * @return string
     */
    private function createUpsertSql(string $table, array $columns, array $ignoreForUpdate = [], int $rowCount = 1): string
    {
        return sprintf(
            '%s ON DUPLICATE KEY UPDATE %s%s',
            $this->createInsertSql($table, $columns, $rowCount),
            implode(', ', array_filter(
                array_map(
                    fn (string $column) => sprintf('`%1$s`=VALUES(`%1$s`)', $column),
                    array_filter($columns, fn (string $column) => ! in_array($column, $ignoreForUpdate))
                )
            )),
            $rowCount === 1 ? ', id=LAST_INSERT_ID(id)' : ''
        );
    }

    /**
     * @param string $identifier
     * @return string
     */
    private function escapeMySqlIdentifier(string $identifier): string
    {
        return '`' . preg_replace('/\W+/', '', $identifier) . '`';
    }

    /**
     * @param array[] $arr
     *
     * @return array
     */
    private function unrollArray(array $arr): array
    {
        $unrolled = [];

        foreach ($arr as $inner) {
            foreach ($inner as $value) {
                $unrolled[] = $value;
            }
        }

        return $unrolled;
    }
}
