<?php
declare(strict_types=1);

namespace CatequilTest\Db;

use Doctrine\DBAL\Exception as DBALException;

/**
 * Class ConnectionTest
 *
 * @package CatequilTest\Db
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-db
 */
final class ConnectionTest extends AbstractDbTestCase
{
    /**
     * @dataProvider multiInsertDataProvider
     * @throws DBALException
     */
    public function testMultiInsert(array $data, string $sql, array $params)
    {
        $conn = $this->getConnection();

        $this->connection()
             ->expects($this->once())
             ->method('prepare')
             ->with($sql)
             ->willReturn($this->statement());

        $this->statement()
             ->expects($this->once())
             ->method('execute')
             ->with($params)
             ->willReturn($this->result());

        $this->result()
             ->expects($this->once())
             ->method('rowCount')
             ->willReturn(count($data));

        $inserted = $conn->multiInsert('test', $data);
        self::assertEquals(count($data), $inserted);
    }

    /**
     * @dataProvider multiUpsertDataProvider
     * @throws DBALException
     */
    public function testMultiUpsert(array $data, string $sql, array $params)
    {
        $conn = $this->getConnection();

        $this->connection()
             ->expects($this->once())
             ->method('prepare')
             ->with($sql)
             ->willReturn($this->statement());

        $this->statement()
             ->expects($this->once())
             ->method('execute')
             ->with($params)
             ->willReturn($this->result());

        $this->result()
             ->expects($this->once())
             ->method('rowCount')
             ->willReturn(count($data));

        $upserted = $conn->multiUpsert('test', $data);
        self::assertEquals(\count($data), $upserted);
    }

    /**
     * @dataProvider  upsertDataProvider
     * @throws DBALException
     */
    public function testUpsert(array $data, string $sql, array $params)
    {
        $conn = $this->getConnection();

        $this->connection()
            ->expects($this->once())
            ->method('prepare')
            ->with($sql)
            ->willReturn($this->statement());

        $this->statement()
            ->expects($this->once())
            ->method('execute')
            ->with($params)
            ->willReturn($this->result());

        $this->result()
            ->expects($this->once())
            ->method('rowCount')
            ->willReturn(1);

        $upserted = $conn->upsert('test', $data);
        self::assertEquals(1, $upserted);
    }

    /**
     * @return void
     * @throws DBALException
     */
    public function testUpsertIgnoresColumns(): void
    {
        $data = [
            'a' => 'av',
            'b' => '0',
            'c' => 'cv',
            'd' => 'dv',
        ];
        /**
         * @noinspection SqlNoDataSourceInspection
         * @noinspection SqlResolve
         */
        $sql = 'INSERT INTO `test` (`a`, `b`, `c`, `d`) VALUES (?, ?, ?, ?) ON DUPLICATE KEY UPDATE `a`=VALUES(`a`), `d`=VALUES(`d`), id=LAST_INSERT_ID(id)';

        $conn = $this->getConnection();

        $this->connection()
             ->expects($this->once())
             ->method('prepare')
             ->with($sql);

        $conn->upsert('test', $data, ['b', 'c']);
    }

    public function multiInsertDataProvider(): array
    {
        /**
         * @noinspection SqlNoDataSourceInspection
         * @noinspection SqlResolve
         */
        return [[
            'data' => [['a' => 'av']],
            'sql' => 'INSERT INTO `test` (`a`) VALUES (?)',
            'params' => ['av'],
        ], [
            'data' => [[
                'a' => 'av',
                'b' => 'bv',
            ], [
                'a' => 'av',
                'b' => 'bv',
            ]],
            'sql' => 'INSERT INTO `test` (`a`, `b`) VALUES (?, ?), (?, ?)',
            'params' => ['av', 'bv', 'av', 'bv'],
        ], [
            'data' => [[
                'a' => 'av',
                'b' => '0',
                'c' => 'cv',
                'd' => 'dv',
            ], [
                'a' => 'av',
                'b' => 'bv',
                'c' => '',
                'd' => 'dv',
            ]],
            'sql' => 'INSERT INTO `test` (`a`, `b`, `c`, `d`) VALUES (?, ?, ?, ?), (?, ?, ?, ?)',
            'params' => ['av', '0', 'cv', 'dv', 'av', 'bv', '', 'dv'],
        ], [
            'data' => [[
                'user' => 'vuser',
                'key' => 'vkey',
                'order' => 'vorder',
                'group' => 'vgroup',
            ]],
            'sql' => 'INSERT INTO `test` (`user`, `key`, `order`, `group`) VALUES (?, ?, ?, ?)',
            'params' => ['vuser', 'vkey', 'vorder', 'vgroup'],
        ]];
    }

    public function multiUpsertDataProvider(): array
    {
        /**
         * @noinspection SqlNoDataSourceInspection
         * @noinspection SqlResolve
         */
        return [[
            'data' => [['a' => 'av']],
            'sql' => 'INSERT INTO `test` (`a`) VALUES (?) ON DUPLICATE KEY UPDATE `a`=VALUES(`a`), id=LAST_INSERT_ID(id)',
            'params' => ['av'],
        ], [
            'data' => [[
                'a' => 'av',
                'b' => 'bv',
            ], [
                'a' => 'av',
                'b' => 'bv',
            ]],
            'sql' => 'INSERT INTO `test` (`a`, `b`) VALUES (?, ?), (?, ?) ON DUPLICATE KEY UPDATE `a`=VALUES(`a`), `b`=VALUES(`b`)',
            'params' => ['av', 'bv', 'av', 'bv'],
        ], [
            'data' => [[
                'a' => 'av',
                'b' => '0',
                'c' => 'cv',
                'd' => 'dv',
            ], [
                'a' => 'av',
                'b' => 'bv',
                'c' => '',
                'd' => 'dv',
            ]],
            'sql' => 'INSERT INTO `test` (`a`, `b`, `c`, `d`) VALUES (?, ?, ?, ?), (?, ?, ?, ?) ON DUPLICATE KEY UPDATE `a`=VALUES(`a`), `b`=VALUES(`b`), `c`=VALUES(`c`), `d`=VALUES(`d`)',
            'params' => ['av', '0', 'cv', 'dv', 'av', 'bv', '', 'dv'],
        ], [
            'data' => [[
                'user' => 'vuser',
                'key' => 'vkey',
                'order' => 'vorder',
                'group' => 'vgroup',
            ]],
            'sql' => 'INSERT INTO `test` (`user`, `key`, `order`, `group`) VALUES (?, ?, ?, ?) ON DUPLICATE KEY UPDATE `user`=VALUES(`user`), `key`=VALUES(`key`), `order`=VALUES(`order`), `group`=VALUES(`group`), id=LAST_INSERT_ID(id)',
            'params' => ['vuser', 'vkey', 'vorder', 'vgroup'],
        ]];
    }

    public function upsertDataProvider(): array
    {
        /**
         * @noinspection SqlNoDataSourceInspection
         * @noinspection SqlResolve
         */
        return [[
            'data' => ['a' => 'av'],
            'sql' => 'INSERT INTO `test` (`a`) VALUES (?) ON DUPLICATE KEY UPDATE `a`=VALUES(`a`), id=LAST_INSERT_ID(id)',
            'params' => ['av'],
        ], [
            'data' => [
                'a' => 'av',
                'b' => 'bv',
            ],
            'sql' => 'INSERT INTO `test` (`a`, `b`) VALUES (?, ?) ON DUPLICATE KEY UPDATE `a`=VALUES(`a`), `b`=VALUES(`b`), id=LAST_INSERT_ID(id)',
            'params' => ['av', 'bv'],
        ], [
            'data' => [
                'a' => 'av',
                'b' => '0',
                'c' => '',
                'd' => 'dv',
            ],
            'sql' => 'INSERT INTO `test` (`a`, `b`, `c`, `d`) VALUES (?, ?, ?, ?) ON DUPLICATE KEY UPDATE `a`=VALUES(`a`), `b`=VALUES(`b`), `c`=VALUES(`c`), `d`=VALUES(`d`), id=LAST_INSERT_ID(id)',
            'params' => ['av', '0', '', 'dv'],
        ], [
            'data' => [
                'user' => 'vuser',
                'key' => 'vkey',
                'order' => 'vorder',
                'group' => 'vgroup',
            ],
            'sql' => 'INSERT INTO `test` (`user`, `key`, `order`, `group`) VALUES (?, ?, ?, ?) ON DUPLICATE KEY UPDATE `user`=VALUES(`user`), `key`=VALUES(`key`), `order`=VALUES(`order`), `group`=VALUES(`group`), id=LAST_INSERT_ID(id)',
            'params' => ['vuser', 'vkey', 'vorder', 'vgroup'],
        ]];
    }
}
